--------------------------------------------------------------------------
-- RessSpeak.lua 
--------------------------------------------------------------------------
--[[
RessSpeak

author: Mike Zhu
bug report to: ressspeak (at) gmail (dot) com

- Say something witty or funny when rezing your fallen buddies

]]--

RS_VERSIONSTR = "1.2.5";
RS_REG = "%s";
RS_DEBUG = "%s: %s";
RS_DEBU2 = "%s: %d";
rsstatus = true;
rsdebug = false;
Resstable = {};
fgtable = {};
classmultiplier = 6;
racemultiplier = 3;
gendermultiplier = 2;
cctalk = false;

RessSpeak_Commands =  {

	["help"] =		"RessSpeak_Help";
	["on"] =		"RessSpeak_On";
	["off"] =		"RessSpeak_Off";
	["speech"] =	"RessSpeak_speak";
	["debug"] =		"RS_DEBUGTOGGLE";
	["class"] =		"RS_classmulti";
	["race"] =		"RS_racemulti";
	["gender"] =	"RS_gendermulti";
	["ccs"] =		"RS_crowdcontrol";

};

isalliance = {
	["human"] =		1;
	["dwarf"] =		1;
	["night elf"] =	1;
	["gnome"] =		1;
	["draenei"] =	1;
};

-- Toggle debug mode
function RS_DEBUGTOGGLE()
	if rsdebug then
		rsdebug = false;
		RSPrint(string.format(RS_REG, "RessSpeak Normal Mode"));
	else
		rsdebug = true;
		RSPrint(string.format(RS_REG, "RessSpeak Debug Mode"));
	end
end

function RS_crowdcontrol(spell)
	cctalk = true;
	RessSpeak_speak(spell);
end

function RessSpeak_OnLoad(self)
	
	SlashCmdList["RESSSPEAK"] = RessSpeak_Handler;
	SLASH_RESSSPEAK1 = "/ressspeak";
	SLASH_RESSSPEAK2 = "/res";
	
end

-- parse the command line
function RessSpeak_Handler(Arg1)
	local _, _, subcom, rest = strfind(Arg1, "[ ]*(%w+)[ ]*(.*)");
	if (subcom) then
		subcom = strlower(subcom);
	else
		RessSpeak_Help();
		return;
	end
	if (RessSpeak_Commands[subcom]) then
		getglobal(RessSpeak_Commands[subcom])(rest);
	else
		RessSpeak_Help();
	end
end

-- Help function
function RessSpeak_Help()
	RSPrint(string.format("%s %s", "RessSpeak Version:", RS_VERSIONSTR));
	RSPrint(string.format(RS_REG, "/ressspeak on - Turns RessSpeak On."));
	RSPrint(string.format(RS_REG, "/ressspeak off - Turns RessSpeak Off."));
	RSPrint(string.format(RS_REG, "/ressspeak speech - Speaks random message!"));
	RSPrint(string.format(RS_REG, "/ressspeak (help) - Displays this message."));
	RSPrint(string.format(RS_REG, "/ressspeak <class/race/gender> number - Changes the corresponding multiplier to the supplied number."));
	RSPrint(string.format(RS_REG, "/ressspeak ccs spellname- Crowd Control instead of ressurection speech!"));
end

-- Turn RessSpeak on
function RessSpeak_On()
	if rsstatus then
		RSPrint(string.format(RS_REG, "RessSpeak already on"));
		return;
	else
		rsstatus = true;
		RSPrint(string.format(RS_REG, "RessSpeak activated"));
	end	
end

-- Turn RessSpeak off
function RessSpeak_Off()
	if rsstatus then
		rsstatus = false;
		RSPrint(string.format(RS_REG, "RessSpeak deactivated"));
		return;
	else
		RSPrint(string.format(RS_REG, "RessSpeak already off"));
	end
end

-- Wrapper for DEFAULT_CHAT_FRAME:AddMessage
function RSPrint(message)
	local r = 0.50;
	local g = 0.50;
	local b = 1.00;
		
	DEFAULT_CHAT_FRAME:AddMessage(message,r,g,b);
end

-- The speak function
function RessSpeak_speak(castspell)
	if not rsstatus then
		RSPrint(string.format(RS_REG, "RessSpeak is off"));
		return;
	end
	
	local srace_loc, srace_eng = UnitRace("player");
	local sgender = UnitSex("player");
	
	if (isalliance[strlower(srace_eng)]) then
		if sgender == 2 then
			fgtable = Alliance_Male;
		else
			fgtable = Alliance_Female;
		end
	else
		if sgender == 2 then
			fgtable = Horde_Male;
		else
			fgtable = Horde_Female;
		end
	end	
	
	local tclass_loc, tclass_eng = UnitClass("playertarget");
	local tname = UnitName("playertarget");
	local tgender = UnitSex("playertarget");
	local trace_loc, trace_eng = UnitRace("playertarget");
	local deathboolean = UnitIsDeadOrGhost("playertarget");
	
	if (tclass_loc == nil) then
		tclass_loc = "none";
		tclass_eng = "NONE";
	end
	if (trace_loc == nil) then
		trace_loc = "none";
		trace_eng = "NONE";
	end
	if (deathboolean == nil) then
		deathboolean = 0;
	end
	if (tname == nil) then
		tname = "No target";
	end
	if (tgender == nil) then
		tgender = 1;
	end
	
	if rsdebug then
		RSPrint(string.format(RS_DEBUG, "Target Class", tclass_loc));
		RSPrint(string.format(RS_DEBUG, "Target Name", tname));
		RSPrint(string.format(RS_DEBUG, "Target Gender", tgender));
		RSPrint(string.format(RS_DEBUG, "Target Race", trace_loc));
		RSPrint(string.format(RS_DEBU2, "Is Dead or Ghost", deathboolean));
	end

	if (tname == "No target") then
		RSPrint(string.format(RS_REG, "You have no target selected, RessSpeak needs a target!"));
		return;
	end
	
	-- make a new table
	if cctalk then
		Resstable = createcctable(castspell);
	else
		Resstable = createreztable(strlower(tclass_loc), strlower(trace_loc), tgender, fgtable);
	end
	
	-- main loop
	if (deathboolean == 1 or rsdebug) then
		local Num_Entries = #(Resstable);
		Num_Entries = Num_Entries / 5;
		math.randomseed(GetTime()*1234);
		local entrynum = math.random(Num_Entries);
		entrynum = entrynum - 1;

		if rsdebug then
			RSPrint(string.format(RS_DEBU2, "Table size", Num_Entries));
			RSPrint(string.format(RS_DEBU2, "Random Number", entrynum));
		end
		
		local index = entrynum * 5;
		
		
		if rsdebug then
			local temp_class = Resstable[index+3];
			local temp_race = Resstable[index+4];
			local temp_gender = Resstable[index+5];
			RSPrint(string.format(RS_DEBUG, "Table Class", temp_class));
			RSPrint(string.format(RS_DEBUG, "Table Race", temp_race));
			RSPrint(string.format(RS_DEBUG, "Table Gender", temp_gender));
		end
		
		local method = Resstable[index+2];
		if not (strlower(method) == "emote") then
			if (not UnitInParty("playertarget") and not UnitInRaid("playertarget")) then
				method = "SAY";
			elseif (not UnitInRaid("playertarget")) then
				method = "PARTY";
			else
				method = "RAID";
			end
		end
		
		if rsdebug then
			RSPrint(string.format(RS_DEBUG, "Method", method));
			RSPrint(string.format(RS_DEBUG, "Message", Resstable[index+1]));
			SendChatMessage(Resstable[index+1],"WHISPER",GetDefaultLanguage("player"),UnitName("player"));
		else
			SendChatMessage(Resstable[index+1],strupper(method));
		end
	end
	if (cctalk or rsdebug) then
		local Num_Entries = #(Resstable);
		Num_Entries = Num_Entries / 3;
		math.randomseed(GetTime()*1234);
		local entrynum = math.random(Num_Entries);
		entrynum = entrynum - 1;

		if rsdebug then
			RSPrint(string.format(RS_DEBU2, "Table size", Num_Entries));
			RSPrint(string.format(RS_DEBU2, "Random Number", entrynum));
		end
		
		local index = entrynum * 3;
		
		if rsdebug then
			local temp_spell = Resstable[index+2];
			RSPrint(string.format(RS_DEBUG, "Table Spell", temp_spell));
		end
		
		local method = Resstable[index+3];
		if not strlower(method) == "emote" then
			if (UnitInRaid("player") == 1) then
				method = "raid";
			else
				method = "party";
			end
		end
		
		if rsdebug then
			RSPrint(string.format(RS_DEBUG, "Method", method));
			RSPrint(string.format(RS_DEBUG, "Message", Resstable[index+1]));
			SendChatMessage(Resstable[index+1],"WHISPER",GetDefaultLanguage("player"),UnitName("player"));
		else
			SendChatMessage(Resstable[index+1],strupper(method));
		end
	end
	cctalk = false;
end

function createcctable(pspell)

	if rsdebug then
		RSPrint(string.format(RS_DEBUG, "Spell Cast", pspell));
	end
	
	local desttable = {};
	local sourcesize = #cctable;
	local sourcenumentry = sourcesize / 3;
	
	if rsdebug then
		RSPrint(string.format(RS_DEBU2, "Source size: ", sourcesize));
	end
	
	local btspell = nil;
	
	for j=1,sourcenumentry do
		insertnumber = 1;
		cur_tableptr = (j - 1) * 3;
		
		btspell = cctable[cur_tableptr + 2];
		
		if (strlower(btspell) == strlower(pspell)) then
			table.insert(desttable, cctable[cur_tableptr + 1]);
			table.insert(desttable, cctable[cur_tableptr + 2]);
			table.insert(desttable, cctable[cur_tableptr + 3]);
		end
	end
	
	return desttable;
end

function createreztable(class, race, gender, basetable)

	if rsdebug then
		RSPrint(string.format(RS_DEBUG, "Class", class));
		RSPrint(string.format(RS_DEBUG, "Race", race));
		RSPrint(string.format(RS_DEBUG, "gender", gender));
	end

	local desttable = {};
	local insertnumber = 1;
	local sourcesize = #basetable;
	local sourcenumentry = sourcesize / 5;
	
	if rsdebug then
		RSPrint(string.format(RS_DEBU2, "Source size: ", sourcesize));
	end
	
	local btclass = nil;
	local btrace = nil;
	local btgender = nil;
	
	
	for j=1,sourcenumentry do
		insertnumber = 1;
		cur_tableptr = (j - 1) * 5;
		
		btclass = basetable[cur_tableptr + 3];
		btrace = basetable[cur_tableptr + 4];
		btgender = basetable[cur_tableptr +5];
		
		if (class == btclass) then
			insertnumber = insertnumber * classmultiplier;
		elseif (btclass ~= "all") then
			insertnumber = 0;
		end
		if (race == btrace) then
			insertnumber = insertnumber * racemultiplier;
		elseif (btrace ~= "all") then
			insertnumber = 0;
		end
		if (gender == btgender) then
			insertnumber = insertnumber * gendermultiplier;
		elseif (btgender ~= "all") then
			insertnumber = 0;
		end
		if rsdebug then
			RSPrint(string.format("%s %d %s %d", "insert: ", insertnumber, "position: ", cur_tableptr));
		end
		desttable = tablencopy(basetable, cur_tableptr, desttable, insertnumber);
	end
		
	return desttable;
	
end

-- copy a set of sayings n times into new table
function tablencopy(source_table, start_val, dest_table, times)
	
	if (times < 1) then
		return dest_table;
	end
	
	for i=1,times do
		table.insert(dest_table,source_table[start_val+1]);
		table.insert(dest_table,source_table[start_val+2]);
		table.insert(dest_table,source_table[start_val+3]);
		table.insert(dest_table,source_table[start_val+4]);
		table.insert(dest_table,source_table[start_val+5]);
	end
	
	return dest_table;

end

function RS_classmulti(newval)
	classmultiplier = tonumber(newval);
	RSPrint(string.format("%s %d", "Class multiplier set to", tonumber(newval)));
end

function RS_racemulti(newval)
	racemultiplier = tonumber(newval);
	RSPrint(string.format("%s %d", "Race multiplier set to", tonumber(newval)));
end

function RS_gendermulti(newval)
	gendermultiplier = tonumber(newval);
	RSPrint(string.format("%s %d", "Gender multiplier set to", tonumber(newval)));
end
